@@ -107,7 +107,7 @@ pub extern "C" fn quiche_version() -> *const u8 {
 
 struct Logger {
     cb: extern "C" fn(line: *const u8, argp: *mut c_void),
-    argp: std::sync::atomic::AtomicPtr<c_void>,
+    argp: atomic::AtomicPtr<c_void>,
 }
 
 impl log::Log for Logger {
@@ -215,6 +215,13 @@ pub extern "C" fn quiche_config_discover_pmtu(config: &mut Config, v: bool) {
     config.discover_pmtu(v);
 }
 
+#[no_mangle]
+pub extern "C" fn quiche_config_set_pmtud_max_probes(
+    config: &mut Config, max_probes: u8,
+) {
+    config.set_pmtud_max_probes(max_probes);
+}
+
 #[no_mangle]
 pub extern "C" fn quiche_config_log_keys(config: &mut Config) {
     config.log_keys();
@@ -621,19 +628,18 @@ pub extern "C" fn quiche_conn_new_with_tls(
         None
     };
 
+    let retry_cids = odcid.as_ref().map(|odcid| RetryConnectionIds {
+        original_destination_cid: odcid,
+        retry_source_cid: &scid,
+    });
+
     let local = std_addr_from_c(local, local_len);
     let peer = std_addr_from_c(peer, peer_len);
 
     let tls = unsafe { tls::Handshake::from_ptr(ssl) };
 
     match Connection::with_tls(
-        &scid,
-        odcid.as_ref(),
-        local,
-        peer,
-        config,
-        tls,
-        is_server,
+        &scid, retry_cids, local, peer, config, tls, is_server,
     ) {
         Ok(c) => Box::into_raw(Box::new(c)),
 
@@ -925,7 +931,7 @@ pub extern "C" fn quiche_conn_stream_shutdown(
 
 #[no_mangle]
 pub extern "C" fn quiche_conn_stream_capacity(
-    conn: &Connection, stream_id: u64,
+    conn: &mut Connection, stream_id: u64,
 ) -> ssize_t {
     match conn.stream_capacity(stream_id) {
         Ok(v) => v as ssize_t,
@@ -1256,17 +1262,27 @@ pub struct Stats {
     recv: usize,
     sent: usize,
     lost: usize,
+    spurious_lost: usize,
     retrans: usize,
     sent_bytes: u64,
     recv_bytes: u64,
     acked_bytes: u64,
     lost_bytes: u64,
     stream_retrans_bytes: u64,
+    dgram_recv: usize,
+    dgram_sent: usize,
     paths_count: usize,
     reset_stream_count_local: u64,
     stopped_stream_count_local: u64,
     reset_stream_count_remote: u64,
     stopped_stream_count_remote: u64,
+    data_blocked_sent_count: u64,
+    stream_data_blocked_sent_count: u64,
+    data_blocked_recv_count: u64,
+    stream_data_blocked_recv_count: u64,
+    path_challenge_rx_count: u64,
+    bytes_in_flight_duration_msec: u64,
+    tx_buffered_inconsistent: bool,
 }
 
 pub struct TransportParams {
@@ -1292,17 +1308,29 @@ pub extern "C" fn quiche_conn_stats(conn: &Connection, out: &mut Stats) {
     out.recv = stats.recv;
     out.sent = stats.sent;
     out.lost = stats.lost;
+    out.spurious_lost = stats.spurious_lost;
     out.retrans = stats.retrans;
     out.sent_bytes = stats.sent_bytes;
     out.recv_bytes = stats.recv_bytes;
     out.acked_bytes = stats.acked_bytes;
     out.lost_bytes = stats.lost_bytes;
     out.stream_retrans_bytes = stats.stream_retrans_bytes;
+    out.dgram_recv = stats.dgram_recv;
+    out.dgram_sent = stats.dgram_sent;
     out.paths_count = stats.paths_count;
     out.reset_stream_count_local = stats.reset_stream_count_local;
     out.stopped_stream_count_local = stats.stopped_stream_count_local;
     out.reset_stream_count_remote = stats.reset_stream_count_remote;
     out.stopped_stream_count_remote = stats.stopped_stream_count_remote;
+    out.data_blocked_sent_count = stats.data_blocked_sent_count;
+    out.stream_data_blocked_sent_count = stats.stream_data_blocked_sent_count;
+    out.data_blocked_recv_count = stats.data_blocked_recv_count;
+    out.stream_data_blocked_recv_count = stats.stream_data_blocked_recv_count;
+    out.path_challenge_rx_count = stats.path_challenge_rx_count;
+    out.bytes_in_flight_duration_msec =
+        stats.bytes_in_flight_duration.as_millis() as u64;
+    out.tx_buffered_inconsistent =
+        stats.tx_buffered_state != TxBufferTrackingState::Ok;
 }
 
 #[no_mangle]
@@ -1349,8 +1377,12 @@ pub struct PathStats {
     sent: usize,
     lost: usize,
     retrans: usize,
+    total_pto_count: usize,
+    dgram_recv: usize,
+    dgram_sent: usize,
     rtt: u64,
     min_rtt: u64,
+    max_rtt: u64,
     rttvar: u64,
     cwnd: usize,
     sent_bytes: u64,
@@ -1359,6 +1391,8 @@ pub struct PathStats {
     stream_retrans_bytes: u64,
     pmtu: usize,
     delivery_rate: u64,
+    max_bandwidth: u64,
+    startup_exit_cwnd: u64,
 }
 
 #[no_mangle]
@@ -1378,6 +1412,9 @@ pub extern "C" fn quiche_conn_path_stats(
     out.sent = stats.sent;
     out.lost = stats.lost;
     out.retrans = stats.retrans;
+    out.total_pto_count = stats.total_pto_count;
+    out.dgram_recv = stats.dgram_recv;
+    out.dgram_sent = stats.dgram_sent;
     out.rtt = stats.rtt.as_nanos() as u64;
     out.min_rtt = stats.min_rtt.unwrap_or_default().as_nanos() as u64;
     out.rttvar = stats.rttvar.as_nanos() as u64;
@@ -1388,6 +1425,9 @@ pub extern "C" fn quiche_conn_path_stats(
     out.stream_retrans_bytes = stats.stream_retrans_bytes;
     out.pmtu = stats.pmtu;
     out.delivery_rate = stats.delivery_rate;
+    out.max_bandwidth = stats.max_bandwidth.unwrap_or(0);
+    out.startup_exit_cwnd =
+        stats.startup_exit.map(|s| s.cwnd as u64).unwrap_or(0);
 
     0
 }
@@ -1899,7 +1939,7 @@ fn optional_std_addr_from_c(
 fn std_addr_from_c(addr: &sockaddr, addr_len: socklen_t) -> SocketAddr {
     match addr.sa_family as _ {
         AF_INET => {
-            assert!(addr_len as usize == std::mem::size_of::<sockaddr_in>());
+            assert!(addr_len as usize == size_of::<sockaddr_in>());
 
             let in4 = unsafe { *(addr as *const _ as *const sockaddr_in) };
 
@@ -1925,7 +1965,7 @@ fn std_addr_from_c(addr: &sockaddr, addr_len: socklen_t) -> SocketAddr {
         },
 
         AF_INET6 => {
-            assert!(addr_len as usize == std::mem::size_of::<sockaddr_in6>());
+            assert!(addr_len as usize == size_of::<sockaddr_in6>());
 
             let in6 = unsafe { *(addr as *const _ as *const sockaddr_in6) };
 
@@ -1960,7 +2000,7 @@ fn std_addr_to_c(addr: &SocketAddr, out: &mut sockaddr_storage) -> socklen_t {
 
     match addr {
         SocketAddr::V4(addr) => unsafe {
-            let sa_len = std::mem::size_of::<sockaddr_in>();
+            let sa_len = size_of::<sockaddr_in>();
             let out_in = out as *mut _ as *mut sockaddr_in;
 
             let s_addr = u32::from_ne_bytes(addr.ip().octets());
@@ -1997,7 +2037,7 @@ fn std_addr_to_c(addr: &SocketAddr, out: &mut sockaddr_storage) -> socklen_t {
         },
 
         SocketAddr::V6(addr) => unsafe {
-            let sa_len = std::mem::size_of::<sockaddr_in6>();
+            let sa_len = size_of::<sockaddr_in6>();
             let out_in6 = out as *mut _ as *mut sockaddr_in6;
 
             #[cfg(not(windows))]
@@ -2045,8 +2085,8 @@ fn std_addr_to_c(addr: &SocketAddr, out: &mut sockaddr_storage) -> socklen_t {
 }
 
 #[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "windows")))]
-fn std_time_to_c(time: &std::time::Instant, out: &mut timespec) {
-    const INSTANT_ZERO: std::time::Instant =
+fn std_time_to_c(time: &Instant, out: &mut timespec) {
+    const INSTANT_ZERO: Instant =
         unsafe { std::mem::transmute(std::time::UNIX_EPOCH) };
 
     let raw_time = time.duration_since(INSTANT_ZERO);
@@ -2056,7 +2096,7 @@ fn std_time_to_c(time: &std::time::Instant, out: &mut timespec) {
 }
 
 #[cfg(any(target_os = "macos", target_os = "ios", target_os = "windows"))]
-fn std_time_to_c(_time: &std::time::Instant, out: &mut timespec) {
+fn std_time_to_c(_time: &Instant, out: &mut timespec) {
     // TODO: implement Instant conversion for systems that don't use timespec.
     out.tv_sec = 0;
     out.tv_nsec = 0;
@@ -2078,10 +2118,10 @@ mod tests {
 
         assert_eq!(
             std_addr_to_c(&addr, &mut out),
-            std::mem::size_of::<sockaddr_in>() as socklen_t
+            size_of::<sockaddr_in>() as socklen_t
         );
 
-        let s = std::ffi::CString::new("ddd.ddd.ddd.ddd").unwrap();
+        let s = ffi::CString::new("ddd.ddd.ddd.ddd").unwrap();
 
         let s = unsafe {
             let in_addr = &out as *const _ as *const sockaddr_in;
@@ -2096,7 +2136,7 @@ mod tests {
                 16,
             );
 
-            std::ffi::CString::from_raw(dst).into_string().unwrap()
+            ffi::CString::from_raw(dst).into_string().unwrap()
         };
 
         assert_eq!(s, "127.0.0.1");
@@ -2104,7 +2144,7 @@ mod tests {
         let addr = unsafe {
             std_addr_from_c(
                 &*(&out as *const _ as *const sockaddr),
-                std::mem::size_of::<sockaddr_in>() as socklen_t,
+                size_of::<sockaddr_in>() as socklen_t,
             )
         };
 
@@ -2121,11 +2161,11 @@ mod tests {
 
         assert_eq!(
             std_addr_to_c(&addr, &mut out),
-            std::mem::size_of::<sockaddr_in6>() as socklen_t
+            size_of::<sockaddr_in6>() as socklen_t
         );
 
-        let s = std::ffi::CString::new("dddd:dddd:dddd:dddd:dddd:dddd:dddd:dddd")
-            .unwrap();
+        let s =
+            ffi::CString::new("dddd:dddd:dddd:dddd:dddd:dddd:dddd:dddd").unwrap();
 
         let s = unsafe {
             let in6_addr = &out as *const _ as *const sockaddr_in6;
@@ -2140,7 +2180,7 @@ mod tests {
                 45,
             );
 
-            std::ffi::CString::from_raw(dst).into_string().unwrap()
+            ffi::CString::from_raw(dst).into_string().unwrap()
         };
 
         assert_eq!(s, "2001:db8:85a3::8a2e:370:7334");
@@ -2148,7 +2188,7 @@ mod tests {
         let addr = unsafe {
             std_addr_from_c(
                 &*(&out as *const _ as *const sockaddr),
-                std::mem::size_of::<sockaddr_in6>() as socklen_t,
+                size_of::<sockaddr_in6>() as socklen_t,
             )
         };
 
