@@ -52,7 +52,6 @@ use crate::frame;
 #[cfg(feature = "qlog")]
 use qlog::events::EventData;
 
-use super::pacer;
 use super::Congestion;
 use crate::recovery::bytes_in_flight::BytesInFlight;
 use crate::recovery::rtt::RttStats;
@@ -66,6 +65,7 @@ use crate::recovery::INITIAL_TIME_THRESHOLD;
 use crate::recovery::MAX_OUTSTANDING_NON_ACK_ELICITING;
 use crate::recovery::MAX_PACKET_THRESHOLD;
 use crate::recovery::MAX_PTO_PROBES_COUNT;
+use crate::recovery::PACKET_REORDER_TIME_THRESHOLD;
 
 #[derive(Default)]
 struct RecoveryEpoch {
@@ -548,12 +548,12 @@ impl RecoveryOps for LegacyRecovery {
                 MAX_OUTSTANDING_NON_ACK_ELICITING
     }
 
-    fn get_acked_frames(&mut self, epoch: Epoch) -> Vec<frame::Frame> {
-        std::mem::take(&mut self.epochs[epoch].acked_frames)
+    fn next_acked_frame(&mut self, epoch: Epoch) -> Option<frame::Frame> {
+        self.epochs[epoch].acked_frames.pop()
     }
 
-    fn get_lost_frames(&mut self, epoch: Epoch) -> Vec<frame::Frame> {
-        std::mem::take(&mut self.epochs[epoch].lost_frames)
+    fn next_lost_frame(&mut self, epoch: Epoch) -> Option<frame::Frame> {
+        self.epochs[epoch].lost_frames.pop()
     }
 
     fn get_largest_acked_on_epoch(&self, epoch: Epoch) -> Option<u64> {
@@ -601,7 +601,6 @@ impl RecoveryOps for LegacyRecovery {
             sent_bytes,
             now,
             &mut pkt,
-            &self.rtt_stats,
             self.bytes_lost,
             in_flight,
         );
@@ -628,9 +627,8 @@ impl RecoveryOps for LegacyRecovery {
         trace!("{trace_id} {self:?}");
     }
 
-    fn get_packet_send_time(&self, _now: Instant) -> Instant {
-        // TODO .max(now)
-        self.congestion.get_packet_send_time()
+    fn get_packet_send_time(&self, now: Instant) -> Instant {
+        now
     }
 
     // `peer_sent_ack_ranges` should not be used without validation.
@@ -658,6 +656,7 @@ impl RecoveryOps for LegacyRecovery {
         if let Some(thresh) = spurious_pkt_thresh {
             self.pkt_thresh =
                 self.pkt_thresh.max(thresh.min(MAX_PACKET_THRESHOLD));
+            self.time_thresh = PACKET_REORDER_TIME_THRESHOLD;
         }
 
         // Undo congestion window update.
@@ -873,6 +872,11 @@ impl RecoveryOps for LegacyRecovery {
         self.congestion.delivery_rate()
     }
 
+    fn max_bandwidth(&self) -> Option<Bandwidth> {
+        // TODO implement
+        None
+    }
+
     /// Statistics from when a CCA first exited the startup phase.
     fn startup_exit(&self) -> Option<StartupExit> {
         self.congestion.ssthresh.startup_exit()
@@ -893,14 +897,6 @@ impl RecoveryOps for LegacyRecovery {
                 self.congestion.initial_congestion_window_packets;
         }
 
-        self.congestion.pacer = pacer::Pacer::new(
-            self.congestion.pacer.enabled(),
-            self.cwnd(),
-            0,
-            new_max_datagram_size,
-            self.congestion.pacer.max_pacing_rate(),
-        );
-
         self.max_datagram_size = new_max_datagram_size;
     }
 
@@ -920,7 +916,6 @@ impl RecoveryOps for LegacyRecovery {
         self.epochs[epoch].in_flight_count
     }
 
-    #[cfg(test)]
     fn bytes_in_flight(&self) -> usize {
         self.bytes_in_flight.get()
     }
@@ -931,7 +926,7 @@ impl RecoveryOps for LegacyRecovery {
 
     #[cfg(test)]
     fn pacing_rate(&self) -> u64 {
-        self.congestion.pacer.rate()
+        0
     }
 
     #[cfg(test)]
@@ -940,8 +935,13 @@ impl RecoveryOps for LegacyRecovery {
     }
 
     #[cfg(test)]
-    fn pkt_thresh(&self) -> u64 {
-        self.pkt_thresh
+    fn pkt_thresh(&self) -> Option<u64> {
+        Some(self.pkt_thresh)
+    }
+
+    #[cfg(test)]
+    fn time_thresh(&self) -> f64 {
+        self.time_thresh
     }
 
     #[cfg(test)]
@@ -1001,7 +1001,7 @@ impl RecoveryOps for LegacyRecovery {
             cwnd: self.cwnd() as u64,
             bytes_in_flight: self.bytes_in_flight.get() as u64,
             ssthresh: Some(self.congestion.ssthresh.get() as u64),
-            pacing_rate: self.congestion.pacer.rate(),
+            ..Default::default()
         };
 
         self.qlog_metrics.maybe_update(qlog_metrics)
@@ -1024,20 +1024,10 @@ impl RecoveryOps for LegacyRecovery {
         self.congestion.send_quantum()
     }
 
-    // TODO tests
     fn get_next_release_time(&self) -> ReleaseDecision {
-        let now = Instant::now();
-        let next_send_time = self.congestion.get_packet_send_time();
-        if next_send_time > now {
-            ReleaseDecision {
-                time: ReleaseTime::At(next_send_time),
-                allow_burst: false,
-            }
-        } else {
-            ReleaseDecision {
-                time: ReleaseTime::Immediate,
-                allow_burst: false,
-            }
+        ReleaseDecision {
+            time: ReleaseTime::Immediate,
+            allow_burst: false,
         }
     }
 
@@ -1071,7 +1061,6 @@ impl std::fmt::Debug for LegacyRecovery {
             self.congestion.congestion_recovery_start_time
         )?;
         write!(f, "{:?} ", self.congestion.delivery_rate)?;
-        write!(f, "pacer={:?} ", self.congestion.pacer)?;
 
         if self.congestion.hystart.enabled() {
             write!(f, "hystart={:?} ", self.congestion.hystart)?;
