@@ -186,6 +186,11 @@ pub(super) struct BBRv2NetworkModel {
     /// Determines whether app limited rounds with no bandwidth growth count
     /// towards the rounds threshold to exit startup.
     ignore_app_limited_for_no_bandwidth_growth: bool,
+
+    /// The most recent send rate from the BandwidthSampler.
+    latest_send_rate: Option<Bandwidth>,
+    /// The most recent ack rate from the BandwidthSampler.
+    latest_ack_rate: Option<Bandwidth>,
 }
 
 impl BBRv2NetworkModel {
@@ -230,9 +235,22 @@ impl BBRv2NetworkModel {
 
             ignore_app_limited_for_no_bandwidth_growth: params
                 .ignore_app_limited_for_no_bandwidth_growth,
+
+            latest_send_rate: None,
+            latest_ack_rate: None,
         }
     }
 
+    #[cfg(feature = "qlog")]
+    pub(super) fn send_rate(&self) -> Option<Bandwidth> {
+        self.latest_send_rate
+    }
+
+    #[cfg(feature = "qlog")]
+    pub(super) fn ack_rate(&self) -> Option<Bandwidth> {
+        self.latest_ack_rate
+    }
+
     pub(super) fn max_ack_height(&self) -> usize {
         self.bandwidth_sampler.max_ack_height().unwrap_or(0)
     }
@@ -348,6 +366,9 @@ impl BBRv2NetworkModel {
             self.min_rtt_filter.update(rtt_sample, event_time);
         }
 
+        self.latest_send_rate = sample.sample_max_send_rate;
+        self.latest_ack_rate = sample.sample_max_ack_rate;
+
         congestion_event.bytes_acked =
             self.total_bytes_acked() - prior_bytes_acked;
         congestion_event.bytes_lost = self.total_bytes_lost() - prior_bytes_lost;
